@@ -739,7 +739,7 @@ issize_t msg_buf_external(msg_t *msg,
 			  usize_t N,
 			  usize_t blocksize)
 {
-  msg_buffer_t *ext, *b, **bb;
+  msg_buffer_t *ext = NULL, *b, **bb;
   size_t i, I;
 
   assert(N <= 128 * 1024);
@@ -1190,6 +1190,7 @@ msg_header_t *error_header_parse(msg_t *msg, msg_pub_t *mo,
   return h;
 }
 
+
 /** Complete this header field and parse next header field.
  *
  * This function completes parsing a multi-field header like @Accept,
@@ -1238,11 +1239,12 @@ issize_t msg_parse_next_field(su_home_t *home, msg_header_t *prev,
   return hc->hc_parse(home, h, s, end - s);
 }
 
+
 /** Decode a message header. */
 msg_header_t *msg_header_d(su_home_t *home, msg_t const *msg, char const *b)
 {
   msg_mclass_t const *mc = msg->m_class;
-  msg_href_t const *hr = mc->mc_unknown;
+  msg_href_t const *hr;
   isize_t n;			/* Length of header contents */
   isize_t name_len, xtra;
   msg_header_t *h;
@@ -1382,7 +1384,6 @@ issize_t msg_extract_payload(msg_t *msg, msg_pub_t *mo,
     usize_t current, rest;
 
     current = msg->m_buffer->mb_size - msg->m_buffer->mb_used;
-    rest = body_len - current;
 
     /* Use all the data from our current buffer */
     msg_buf_used(msg, current);
@@ -1678,7 +1679,7 @@ size_t msg_header_prepare(msg_mclass_t const *mc, int flags,
     n += m;
 
     if (hc->hc_name) {
-      if (!comma_list || !next || next == *return_next)
+      if (!hc->hc_name[0] || !comma_list || !next || next == *return_next)
 	s = CRLF, m = 2;
       /* Else encode continuation */
       else if (compact)
@@ -2015,7 +2016,7 @@ msg_header_t **serialize_one(msg_t *msg, msg_header_t *h, msg_header_t **prev)
     for (last = h; last->sh_succ; last = last->sh_succ) {
       /* Ensure that chain is connected */
       assert(last->sh_next == last->sh_succ);
-      assert(last->sh_succ->sh_prev = &last->sh_succ);
+      assert(last->sh_succ->sh_prev == &last->sh_succ);
     }
     prev = &last->sh_succ;
   }
@@ -2468,8 +2469,6 @@ int msg_header_prepend(msg_t *msg,
 msg_header_t **
 msg_hclass_offset(msg_mclass_t const *mc, msg_pub_t const *mo, msg_hclass_t *hc)
 {
-  int i;
-
   assert(mc && hc);
 
   if (mc == NULL || hc == NULL)
@@ -2481,12 +2480,16 @@ msg_hclass_offset(msg_mclass_t const *mc, msg_pub_t const *mo, msg_hclass_t *hc)
       if (mc->mc_hash[j].hr_class == hc) {
 	return (msg_header_t **)((char *)mo + mc->mc_hash[j].hr_offset);
       }
-  }
-  else
+  } else {
     /* Header has no name. */
-    for (i = 0; i <= 6; i++)
-      if (hc->hc_hash == mc->mc_request[i].hr_class->hc_hash)
-	return (msg_header_t **)((char *)mo + mc->mc_request[i].hr_offset);
+    if (hc->hc_hash == mc->mc_request[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_request[0].hr_offset);
+    if (hc->hc_hash == mc->mc_status[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_status[0].hr_offset);
+    if (hc->hc_hash == mc->mc_separator[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_separator[0].hr_offset);
+    if (hc->hc_hash == mc->mc_payload[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_payload[0].hr_offset);
+    if (hc->hc_hash == mc->mc_unknown[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_unknown[0].hr_offset);
+    if (hc->hc_hash == mc->mc_error[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_error[0].hr_offset);
+    if (hc->hc_hash == mc->mc_multipart[0].hr_class->hc_hash) return (msg_header_t **)((char *)mo + mc->mc_multipart[0].hr_offset);
+  }
 
   return NULL;
 }
@@ -2914,7 +2917,6 @@ int msg_header_remove(msg_t *msg, msg_pub_t *pub, msg_header_t *h)
   if (msg == NULL || h == NULL || h == MSG_HEADER_NONE ||
       h->sh_class == NULL)
     return -1;
-
   if (pub == NULL)
     pub = msg->m_object;
 
@@ -3005,8 +3007,8 @@ int msg_header_remove_all(msg_t *msg, msg_pub_t *pub, msg_header_t *h)
  *
  * @param msg message object owning the fragment chain
  * @param pub public message structure to which header is added
- * @param replaced   old header to be removed (may be NULL)
- * @param h  list of header(s) to be added (may be NULL)
+ * @param replaced   old header to be removed
+ * @param h   list of header(s) to be added
  */
 int msg_header_replace(msg_t *msg,
 		       msg_pub_t *pub,
@@ -3015,20 +3017,16 @@ int msg_header_replace(msg_t *msg,
 {
   msg_header_t *h0, *last, **hh, **hh0;
 
-  if (msg == NULL)
+  if (msg == NULL || replaced == NULL)
     return -1;
-
   if (h == NULL || h == MSG_HEADER_NONE || h->sh_class == NULL)
     return msg_header_remove(msg, pub, replaced);
-
   if (pub == NULL)
     pub = msg->m_object;
 
   hh = hh0 = msg_hclass_offset(msg->m_class, pub, h->sh_class);
-
   if (hh == NULL)
     return -1;
-
   if (replaced == NULL)
     return msg_header_add(msg, pub, hh, h);
 
