@@ -63,7 +63,7 @@ const unsigned sdp_struct_align_ = sizeof(void *) - STRUCT_ALIGN_;
 
 
 #define STRUCT_DUP(p, dst, src) \
-  ASSERT_STRUCT_ALIGN(p); assert(*(int*)(src) >= (int)sizeof(*src));	\
+  ASSERT_STRUCT_ALIGN(p); \
   ((*(int*)(src) >= (int)sizeof(*src)					\
     ? (dst = memcpy((p), (src), sizeof(*src)))				\
     : (dst = memcpy((p), (src), *(int*)(src))),				\
@@ -1189,8 +1189,8 @@ int sdp_session_cmp(sdp_session_t const *a, sdp_session_t const *b)
 
   for (ab = a->sdp_bandwidths, bb = b->sdp_bandwidths;
        ab || bb;
-       ab = ab->b_next, bb = bb->b_next)
-    if ((rv = sdp_bandwidth_cmp(a->sdp_bandwidths, b->sdp_bandwidths)))
+       ab = (ab ? ab->b_next : NULL), bb = (bb ? bb->b_next : NULL))
+    if ((rv = sdp_bandwidth_cmp(ab, bb)))
       return rv;
 
   if ((rv = sdp_time_cmp(a->sdp_time, b->sdp_time)))
@@ -1199,14 +1199,14 @@ int sdp_session_cmp(sdp_session_t const *a, sdp_session_t const *b)
     return rv;
 
   for (aa = a->sdp_attributes, ba = b->sdp_attributes;
-       aa || bb;
-       aa = aa->a_next, ba = ba->a_next)
+       aa || ba;
+       aa = (aa ? aa->a_next : NULL), ba = (ba ? ba->a_next : NULL))
     if ((rv = sdp_attribute_cmp(aa, ba)))
       return rv;
 
   for (am = a->sdp_media, bm = b->sdp_media;
        am || bm;
-       am = am->m_next, bm = bm->m_next)
+       am = (am ? am->m_next : NULL), bm = (bm ? bm->m_next : NULL))
     if ((rv = sdp_media_cmp(am, bm)))
       return rv;
 
@@ -1244,6 +1244,8 @@ int sdp_connection_cmp(sdp_connection_t const *a, sdp_connection_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1; 
   if (a->c_nettype != b->c_nettype)
     return a->c_nettype < b->c_nettype ? -1 : 1;
   if (a->c_addrtype != b->c_addrtype)
@@ -1266,6 +1268,9 @@ int sdp_bandwidth_cmp(sdp_bandwidth_t const *a, sdp_bandwidth_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1;
+
   if (a->b_modifier != b->b_modifier)
     return a->b_modifier < b->b_modifier ? -1 : 1;
   if (a->b_modifier == sdp_bw_x &&
@@ -1308,6 +1313,9 @@ int sdp_repeat_cmp(sdp_repeat_t const *a, sdp_repeat_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1;
+
   if (a->r_interval != b->r_interval)
     return a->r_interval < b->r_interval ? -1 : 1;
   if (a->r_duration != b->r_duration)
@@ -1334,6 +1342,8 @@ int sdp_zone_cmp(sdp_zone_t const *a, sdp_zone_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1;
   n = a->z_number_of_adjustments < b->z_number_of_adjustments
     ? a->z_number_of_adjustments : b->z_number_of_adjustments;
   for (i = 0; i < n; i++) {
@@ -1360,6 +1370,9 @@ int sdp_key_cmp(sdp_key_t const *a, sdp_key_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1;
+
   if (a->k_method != b->k_method)
     return a->k_method < b->k_method ? -1 : 1;
   if (a->k_method == sdp_key_x &&
@@ -1393,6 +1406,9 @@ int sdp_rtpmap_cmp(sdp_rtpmap_t const *a, sdp_rtpmap_t const *b)
   if ((a != NULL) != (b != NULL))
     return (a != NULL) < (b != NULL) ? -1 : 1;
 
+  if (!a || !b)
+    return -1;
+
   if (a->rm_pt != b->rm_pt)
     return a->rm_pt < b->rm_pt ? -1 : 1;
 
@@ -1452,6 +1468,9 @@ int sdp_media_cmp(sdp_media_t const *a, sdp_media_t const *b)
   if ((rv = (a != NULL) - (b != NULL)))
     return rv;
 
+  if (!a || !b)
+    return -1;
+
   if (a->m_type != b->m_type)
     return a->m_type < b->m_type ? -1 : 1;
   if (a->m_type == sdp_media_x)
@@ -1478,7 +1497,7 @@ int sdp_media_cmp(sdp_media_t const *a, sdp_media_t const *b)
 
   for (arm = a->m_rtpmaps, brm = b->m_rtpmaps;
        arm || brm;
-       arm = arm->rm_next, brm = brm->rm_next)
+       arm = (arm ? arm->rm_next : NULL), brm = (brm ? brm->rm_next : NULL))
     if ((rv = sdp_rtpmap_cmp(arm, brm)))
       return rv;
 
@@ -1490,22 +1509,22 @@ int sdp_media_cmp(sdp_media_t const *a, sdp_media_t const *b)
 
   for (ac = a->m_connections, bc = b->m_connections;
        ac || bc;
-       ac = ac->c_next, bc = bc->c_next)
+       ac = (ac ? ac->c_next : NULL), bc = (bc ? bc->c_next : NULL))
   if ((rv = sdp_connection_cmp(ac, bc)))
     return rv;
 
   for (ab = a->m_bandwidths, bb = b->m_bandwidths;
        ab || bb;
-       ab = ab->b_next, bb = bb->b_next)
-    if ((rv = sdp_bandwidth_cmp(a->m_bandwidths, b->m_bandwidths)))
+       ab = (ab ? ab->b_next : NULL), bb = (bb ? bb->b_next : NULL))
+    if ((rv = sdp_bandwidth_cmp(ab, bb)))
       return rv;
 
   if ((rv = sdp_key_cmp(a->m_key, b->m_key)))
     return rv;
 
   for (aa = a->m_attributes, ba = b->m_attributes;
-       aa || bb;
-       aa = aa->a_next, ba = ba->a_next)
+       aa || ba;
+       aa = (aa ? aa->a_next : NULL), ba = (ba ? ba->a_next : NULL))
     if ((rv = sdp_attribute_cmp(aa, ba)))
       return rv;
 
@@ -1831,7 +1850,7 @@ int sdp_media_uses_rtp(sdp_media_t const *m)
 {
   return m &&
     (m->m_proto == sdp_proto_rtp ||
-     m->m_proto == sdp_proto_srtp ||
+     m->m_proto == sdp_proto_srtp || m->m_proto == sdp_proto_extended_srtp ||
      (m->m_proto == sdp_proto_x && m->m_proto_name &&
       su_casenmatch(m->m_proto_name, "RTP/", 4)));
 }
@@ -1858,7 +1877,9 @@ int sdp_rtpmap_match(sdp_rtpmap_t const *a, sdp_rtpmap_t const *b)
   if (aparam == bparam)
     return 1;
 
-  if (!aparam) aparam = "1"; if (!bparam) bparam = "1";
+  if (!aparam) aparam = "1";
+
+  if (!bparam) bparam = "1";
 
   if (!su_casematch(aparam, bparam))
     return 0;
@@ -1866,23 +1887,20 @@ int sdp_rtpmap_match(sdp_rtpmap_t const *a, sdp_rtpmap_t const *b)
   return 1;
 }
 
-/** Search for matching a rtpmap entry from list.
+/** Search for matching rtpmap from list.
  *
  * @note
  * The a=fmtp: for the codecs are not compared.
- *
- * @return
- * Pointer to first sdp_rtpmap_t entry on the @a list matching with any
- * sdp_rtpmap_t entry on @a rm.
  */
 sdp_rtpmap_t *sdp_rtpmap_find_matching(sdp_rtpmap_t const *list,
 				       sdp_rtpmap_t const *rm)
 {
   char const *lparam, *rparam;
+  sdp_rtpmap_t const *cp_list = NULL;
 
   if (rm == NULL)
     return NULL;
-
+    
   for (; list; list = list->rm_next) {
     if (rm->rm_rate != list->rm_rate)
       continue;
@@ -1892,15 +1910,21 @@ sdp_rtpmap_t *sdp_rtpmap_find_matching(sdp_rtpmap_t const *list,
 
     lparam = rm->rm_params; rparam = list->rm_params;
 
-    if (lparam == rparam)
-      break;
+    if (lparam == rparam) {
+          cp_list = list;
+          if (rm->rm_pt != list->rm_pt) continue;
+          break;
+    }
+
+    if (!lparam) lparam = "1";
+
+    if (!rparam) rparam = "1";
 
-    if (!lparam) lparam = "1"; if (!rparam) rparam = "1";
     if (!su_casematch(lparam, rparam))
       continue;
 
     break;
   }
 
-  return (sdp_rtpmap_t *)list;
+  return cp_list ? (sdp_rtpmap_t *) cp_list : (sdp_rtpmap_t *)list;
 }
