@@ -64,6 +64,7 @@ char const sip_version_2_0[] = "SIP/2.0";
 extern msg_mclass_t sip_mclass[];
 
 static msg_mclass_t const *_default = sip_mclass;
+static msg_mclass_t *_default_parser_cloned = NULL;
 
 /** Return a built-in SIP parser object. */
 msg_mclass_t const *sip_default_mclass(void)
@@ -71,6 +72,38 @@ msg_mclass_t const *sip_default_mclass(void)
   return _default;
 }
 
+/** Restore default SIP parser to non-extended */
+void sip_default_mclass_restore() {
+	_default = sip_mclass;
+}
+
+/** Destroy SIP parser object */
+void sip_destroy_mclass(msg_mclass_t *mclass) {
+	if (mclass && mclass != sip_mclass) {
+		if (mclass == _default_parser_cloned) {
+			sip_cloned_parser_destroy();
+		} else {
+			if (mclass == _default) {
+				sip_default_mclass_restore();
+			}
+			free(mclass);
+		}
+	}
+}
+
+/** Release SIP parser object if it was cloned. */
+void sip_cloned_parser_destroy(void)
+{
+	if (_default_parser_cloned) {
+		if (_default_parser_cloned == _default) {
+			sip_default_mclass_restore();
+		}
+
+		free(_default_parser_cloned);
+		_default_parser_cloned = NULL;
+	}
+}
+
 /** Update the default SIP parser.
  *
  * Use the extended SIP parser as default one.
@@ -84,6 +117,11 @@ msg_mclass_t const *sip_default_mclass(void)
  *     exit(2);
  *   }
  * @endcode
+ * When finish don't forget to call 
+ * @code
+ *   sip_destroy_mclass(sip_default_mclass());
+ * @endcode
+ * as return of sip_extend_mclass(NULL) requires to be freed.
  *
  * The default parser is not extended because it may break the old
  * applications looking for extension headers from sip_unknown list.
@@ -128,10 +166,12 @@ msg_mclass_t *sip_extend_mclass(msg_mclass_t *input)
 {
   msg_mclass_t *mclass;
 
-  if (input == NULL || input == _default)
-    mclass = msg_mclass_clone(_default, 0, 0);
-  else
+  if (input == NULL || input == _default) {
+    _default_parser_cloned = msg_mclass_clone(_default, 0, 0);
+    mclass = _default_parser_cloned;
+  } else {
     mclass = input;
+  }
 
   if (mclass) {
     extern msg_hclass_t * const sip_extensions[];
@@ -143,9 +183,10 @@ msg_mclass_t *sip_extend_mclass(msg_mclass_t *input)
 	continue;
 
       if (msg_mclass_insert_header(mclass, hclass, 0) < 0) {
-	if (input != mclass)
-	  free(mclass);
-	return mclass = NULL;
+        if (input != mclass) {
+          sip_destroy_mclass(mclass);
+        }
+        return mclass = NULL;
       }
     }
   }
@@ -451,6 +492,8 @@ sip_method_t sip_method_code(char const *name)
 char const sip_transport_udp[] = "SIP/2.0/UDP";
 char const sip_transport_tcp[] = "SIP/2.0/TCP";
 char const sip_transport_sctp[] = "SIP/2.0/SCTP";
+char const sip_transport_ws[] = "SIP/2.0/WS";
+char const sip_transport_wss[] = "SIP/2.0/WSS";
 char const sip_transport_tls[] = "SIP/2.0/TLS";
 
 /** Decode transport */
@@ -470,6 +513,8 @@ issize_t sip_transport_d(char **ss, char const **ttransport)
       (!TRANSPORT_MATCH(sip_transport_udp) &&
        !TRANSPORT_MATCH(sip_transport_tcp) &&
        !TRANSPORT_MATCH(sip_transport_sctp) &&
+       !TRANSPORT_MATCH(sip_transport_ws) &&
+       !TRANSPORT_MATCH(sip_transport_wss) &&
        !TRANSPORT_MATCH(sip_transport_tls))) {
     /* Protocol name */
     transport = pn = s;
@@ -512,6 +557,10 @@ issize_t sip_transport_d(char **ss, char const **ttransport)
 	transport = sip_transport_tcp;
       else if (su_casematch(transport, sip_transport_sctp))
 	transport = sip_transport_sctp;
+      else if (su_casematch(transport, sip_transport_ws))
+	transport = sip_transport_ws;
+      else if (su_casematch(transport, sip_transport_wss))
+	transport = sip_transport_wss;
       else if (su_casematch(transport, sip_transport_tls))
 	transport = sip_transport_tls;
     }
@@ -529,10 +578,14 @@ isize_t sip_transport_xtra(char const *transport)
   if (transport == sip_transport_udp ||
       transport == sip_transport_tcp ||
       transport == sip_transport_sctp ||
+      transport == sip_transport_ws ||
+      transport == sip_transport_wss ||
       transport == sip_transport_tls ||
       su_casematch(transport, sip_transport_udp) ||
       su_casematch(transport, sip_transport_tcp) ||
       su_casematch(transport, sip_transport_sctp) ||
+      su_casematch(transport, sip_transport_ws) ||
+      su_casematch(transport, sip_transport_wss) ||
       su_casematch(transport, sip_transport_tls))
     return 0;
 
@@ -550,6 +603,10 @@ void sip_transport_dup(char **pp, char const **dd, char const *s)
     *dd = s;
   else if (s == sip_transport_tls)
     *dd = s;
+  else if (s == sip_transport_ws)
+    *dd = s;
+  else if (s == sip_transport_wss)
+    *dd = s;
   else if (su_casematch(s, sip_transport_udp))
     *dd = sip_transport_udp;
   else if (su_casematch(s, sip_transport_tcp))
@@ -558,6 +615,10 @@ void sip_transport_dup(char **pp, char const **dd, char const *s)
     *dd = sip_transport_sctp;
   else if (su_casematch(s, sip_transport_tls))
     *dd = sip_transport_tls;
+  else if (su_casematch(s, sip_transport_ws))
+    *dd = sip_transport_ws;
+  else if (su_casematch(s, sip_transport_wss))
+    *dd = sip_transport_wss;
   else
     MSG_STRING_DUP(*pp, *dd, s);
 }
@@ -641,7 +702,7 @@ int sip_complete_message(msg_t *msg)
   else {
     if (sip->sip_content_length->l_length != len) {
       sip->sip_content_length->l_length = (uint32_t)len;
-      msg_fragment_clear(sip->sip_content_length->l_common);
+      sip_fragment_clear(sip->sip_content_length->l_common);
     }
   }
 
