@@ -73,6 +73,8 @@ int su_base_port_init(su_port_t *self, su_port_vtable_t const *vtable)
   if (self) {
     self->sup_vtable = vtable;
     self->sup_tail = &self->sup_head;
+    self->sup_max_defer = 15 * 1000;
+
     return su_port_obtain(self);
   }
 
@@ -149,7 +151,6 @@ struct _GSource *su_base_port_gsource(su_port_t *self)
 
 /** @internal Send a message to the port.
  *
- * @retval 1 if port thread needs to be woken
  * @retval 0 if there are other messages in queue, too
  * @retval -1 upon an error
  */
@@ -167,7 +168,10 @@ int su_base_port_send(su_port_t *self, su_msg_r rmsg)
 
     su_port_unlock(self, "su_port_send");
 
-    return wakeup;
+    if (wakeup > 0)
+      su_port_wakeup(self);
+
+    return 0;
   }
   else {
     su_msg_destroy(rmsg);
@@ -317,12 +321,12 @@ int su_base_port_multishot(su_port_t *self, int multishot)
  */
 void su_base_port_run(su_port_t *self)
 {
-  su_duration_t tout = 0;
+  su_duration_t tout = 0, tout2 = 0;
 
   assert(su_port_own_thread(self));
 
   for (self->sup_running = 1; self->sup_running;) {
-    tout = 2000;
+    tout = self->sup_max_defer;
 
     if (self->sup_prepoll)
       self->sup_prepoll(self->sup_pp_magic, self->sup_pp_root);
@@ -330,8 +334,11 @@ void su_base_port_run(su_port_t *self)
     if (self->sup_head)
       self->sup_vtable->su_port_getmsgs(self);
 
-    if (self->sup_timers)
-      su_timer_expire(&self->sup_timers, &tout, su_now());
+    if (self->sup_timers || self->sup_deferrable) {
+      su_time_t now = su_now();
+      su_timer_expire(&self->sup_timers, &tout, now);
+      su_timer_expire(&self->sup_deferrable, &tout2, now);
+    }
 
     if (!self->sup_running)
       break;
@@ -349,13 +356,13 @@ void su_base_port_run_tune(su_port_t *self)
 {
   int i;
   int timers = 0, messages = 0, events = 0;
-  su_duration_t tout = 0, tout0;
+  su_duration_t tout = 0, tout2 = 0;
   su_time_t started = su_now(), woken = started, bedtime = woken;
 
   assert(su_port_own_thread(self));
 
   for (self->sup_running = 1; self->sup_running;) {
-    tout = 2000;
+    tout = self->sup_max_defer;
 
     timers = 0, messages = 0;
 
@@ -365,8 +372,12 @@ void su_base_port_run_tune(su_port_t *self)
     if (self->sup_head)
       messages = self->sup_vtable->su_port_getmsgs(self);
 
-    if (self->sup_timers)
-      timers = su_timer_expire(&self->sup_timers, &tout, su_now());
+    if (self->sup_timers || self->sup_deferrable) {
+      su_time_t now = su_now();
+      timers =
+	su_timer_expire(&self->sup_timers, &tout, now) +
+	su_timer_expire(&self->sup_deferrable, &tout2, now);
+    }
 
     if (!self->sup_running)
       break;
@@ -404,6 +415,16 @@ void su_base_port_break(su_port_t *self)
   self->sup_running = 0;
 }
 
+/** @internal
+ * Check if port is running.
+ *
+ * @param self     pointer to port
+ */
+int su_base_port_is_running(su_port_t const *self)
+{
+  return self->sup_running != 0;
+}
+
 /** @internal Block until wait object is signaled or timeout.
  *
  * This function waits for wait objects and the timers associated with
@@ -435,6 +456,10 @@ su_duration_t su_base_port_step(su_port_t *self, su_duration_t tout)
   if (self->sup_timers)
     su_timer_expire(&self->sup_timers, &tout, now);
 
+  /* XXX: why isn't the timeout ignored here? */
+  if (self->sup_deferrable)
+    su_timer_expire(&self->sup_deferrable, &tout, now);
+
   /* if there are messages do a quick wait */
   if (self->sup_head)
     tout = 0;
@@ -444,15 +469,27 @@ su_duration_t su_base_port_step(su_port_t *self, su_duration_t tout)
   else
     tout = SU_WAIT_FOREVER;
 
-  if (self->sup_head)
+  if (self->sup_head) {
     if (self->sup_vtable->su_port_getmsgs(self)) {
       /* Check for wait events that may have been generated by messages */
       if (self->sup_vtable->su_port_wait_events(self, 0))
 	tout = 0;
     }
+  }
 
-  if (self->sup_timers)
-    su_timer_expire(&self->sup_timers, &tout, su_now());
+  if (self->sup_timers || self->sup_deferrable) {
+    su_duration_t tout2 = SU_WAIT_FOREVER;
+
+    now = su_now();
+    su_timer_expire(&self->sup_timers, &tout, now);
+    su_timer_expire(&self->sup_deferrable, &tout2, now);
+
+    if (tout == SU_WAIT_FOREVER && tout2 != SU_WAIT_FOREVER) {
+      if (tout2 < self->sup_max_defer)
+	tout2 = self->sup_max_defer;
+      tout = tout2;
+    }
+  }
 
   if (self->sup_head)
     tout = 0;
@@ -501,6 +538,22 @@ su_timer_queue_t *su_base_port_timers(su_port_t *self)
   return &self->sup_timers;
 }
 
+su_timer_queue_t *su_base_port_deferrable(su_port_t *self)
+{
+  return &self->sup_deferrable;
+}
+
+int su_base_port_max_defer(su_port_t *self,
+			   su_duration_t *return_duration,
+			   su_duration_t *set_duration)
+{
+  if (set_duration && *set_duration > 0)
+    self->sup_max_defer = *set_duration;
+  if (return_duration)
+    *return_duration = self->sup_max_defer;
+  return 0;
+}
+
 /* ======================================================================
  * Clones
  */
