@@ -257,6 +257,7 @@ int test_sip(void)
   BEGIN();
 
   TEST_S(url_scheme(url_sip), "sip");
+  TEST_S(url_scheme(url_urn), "urn");
   TEST_S(url_scheme(url_sips), "sips");
 
   memset(url, 255, sizeof url);
@@ -341,6 +342,9 @@ int test_sip(void)
   u = url_hdup(home, (url_t*)"SIP:test@127.0.0.1:55"); TEST_1(u);
   TEST(u->url_type, url_sip);
 
+  u = url_hdup(home, (url_t*)"SIP:test@127.0.0.1:"); TEST_1(u);
+  TEST(u->url_type, url_sip);
+
   TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1::55"), NULL);
   TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1:55:"), NULL);
   TEST_P(url_hdup(home, (url_t*)"sip:test@127.0.0.1:sip"), NULL);
